// program implementing k-means algorithm using Weight class

#include <math.h>
#include <fstream.h>

#include "ezsample.hpp"		// includes "weight.hpp"

main( int argc, char ** argv ) {

  if ( argc == 0 ) {
    cerr<< "Usage: vqtest <filename>\n";
    exit(1);
  }
  const unsigned vecDim = 2;
  const unsigned numClasses = 5;
  ezSample<float> sample( argv[1], vecDim );

  unsigned sampleSize = sample.getSize();


  // dim class array
  unsigned* classArr = new unsigned[ sampleSize ];

  // inicialization: 5 classes
  Weight<float> dict[numClasses], auxDict[numClasses];	 // default constructor
  unsigned numElements[numClasses];
  float zero[vecDim] = {0.0, 0.0};

  cout << "Initial Vectors " << endl;
  mysrand();
  for( unsigned i = 0; i < numClasses; i ++ ) {
    float aux[vecDim];
    for ( unsigned j = 0; j < vecDim; j ++ ) 
      aux[j] = (random() % 1000)/500.0 - 1;

    dict[i].setDim( vecDim );
    auxDict[i].setDim( vecDim );
    dict[i] = aux;
    cout << dict[i] << endl;
    
  }

  float oldDistortion = 0;
  float distortion = 1000.0;
  unsigned idx = 1;

  while ( distortion != oldDistortion ) {
    oldDistortion = distortion;
    distortion = 0.0;
    ofstream fileC[numClasses];
     for ( i = 0; i < numClasses; i ++ ) {
       char fileName[40];
       auxDict[i] = zero;
       numElements[i] = 0;
       sprintf( fileName, "clas#%d.%d", idx, i );
       fileC[i].open( fileName );
     }
    idx++;

    // compute classes and distortion
    for ( unsigned k = 0; k < sampleSize; k++ ) {
      Weight<float> auxW;
      auxW = sample.getVect( k );
      float minDist = 1000.0;
      unsigned minVec = numClasses + 10;
      for ( i = 0; i < numClasses; i ++ ) {
	float dist = dict[i].distE( auxW );
	if ( dist < minDist ) {
	  minVec = i;
	  minDist = dist;
	}
      }
      
      classArr[k] = minVec;
      distortion += minDist;
      auxDict[ minVec ] += auxW;
      numElements[minVec]++;
      fileC[minVec] <<  auxW << endl;

    }

    cout << "Step " << idx << " Distortion" << distortion << endl;
    for ( i = 0; i < numClasses; i ++ ) {
      dict[i] = auxDict[i] / numElements[i];    
      fileC[i].close();    
    }

  }
  return(0);


}

  
