// -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8;  comment-column: 35 -*-
//                             BITGENE.CC	
// Implementation of some functions for bitGene class

#include "bitgene.hpp"                    


const double LOG2 =0.30102999566398119521;

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
bitGene::bitGene(const bitGene& bitGene1, 
		 mutRate_t _mutationRate, 
		 genLength_t _genTr1, 
		 genLength_t _genTr2) 
	: rawGene( bitGene1, _mutationRate), 
	  bitSize( bitGene1.bitSize ), 
	  numLoci( bitGene1.numLoci ) 
{
// -- Constructor; only works if _genTr* are different
//    from 0
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

    if ( _genTr1 || _genTr2 ) {
	transposeChunk( _genTr1*bitSize, _genTr2* bitSize, bitSize );
    }  
	  
}

// --------------------------------------------------------------------
bitGene::bitGene(const bitGene& _bitGene,  
		 genOpMode _modeOp, 
		 genLength_t _geneIdx, 
		 mutRate_t _mutationRate,
		 genLength_t _geneIdx2 )
	:rawGene( _bitGene, _mutationRate ), 
	 bitSize( _bitGene.bitSize ), 
	 numLoci( _bitGene.numLoci ) 	{
// --	creates a new genome using unary genetic operators; uses vars as
//	needed, _mutationRate is used always. _geneIdx2 is used only
//      for transposition
// ---------------------------------------------------------------------

    // now applies unary genetic operator as specified to the newly
    // created genome. If mode == CLONE, do nothing; already done.
    if ( _modeOp == TRANSP ) {
	if ( _geneIdx != _geneIdx2 ) 
	  transposeChunk(_geneIdx * bitSize, 
			 _geneIdx2 * bitSize,
			 bitSize);
    } else 
	cerr << "Wrong mode for bitGene in ctor\n";
    
   
}


// --------------------------------------
unsigned bitGene::bit ( unsigned  _idx ) const {
// -- _idx is the bit index, returns 0 or 1
// ----------------------------------------
    unsigned bitInByte = _idx% BITSINBYTE;
    unsigned idxByte = _idx / BITSINBYTE;
    return getStr(idxByte)&&Wgts[bitInByte];
}


// ----------------------------------------------------------------
unsigned bitGene::getValue ( genLength_t _idx ) const {
// -- Extract value of gene number _idx
// -----------------------------------------------------------------
    
    int startPoint = _idx*bitSize;
    int endPoint = (_idx+1)*bitSize;
    
    long Weight = 1;		    // multiply numbers by this
    long result = 0;

    for ( int i = endPoint -1; i >= startPoint; --i ) {
	unsigned bitInByte = i% BITSINBYTE;
	unsigned idxByte = i / BITSINBYTE;
	result += 
	  Weight *((getStr(idxByte)&Wgts[bitInByte+1]) >>(LASTBYTEN -bitInByte) );
	Weight <<=1;
    }

    return result;
}


//-----------------------------------------------------------------------
// Conversion operators
//---------------------

//-----------------------------------------------------
bitGene:: operator unsigned * ( void ) const {
// -- Standard conversion to unsigned vector
//-----------------------------------------------------

    unsigned * tmpArr = new unsigned[ numLoci ];
    for ( unsigned i = 0; i < numLoci; i ++ )
      tmpArr[i] = getValue( i );
    return tmpArr;
}


//-----------------------------------------------------
bitGene::operator char * ( void ) const {
// -- Standard conversion to a string of digits 
//-----------------------------------------------------

    // for each locus, compute number of chars needed
    // to hold the figure
    unsigned numCyp = unsigned( ceil( bitSize * LOG2 ) );
    char * cyph = new char[ numCyp + 1 ];

    char *tmpStr = new char[ numLoci * numCyp + 1];
    strcpy( tmpStr, "" );
    for ( unsigned i = 0; i < numLoci; i ++ ) {
	sprintf( cyph, "%d", getValue(i) );
	strcat( tmpStr, cyph );
    }
    delete cyph;		   // Get rid of it
    return tmpStr;
}
    

// -----------------------------------------------------------------------
// Friendly operators
// ------------------

// -----------------------------------------------------------------------
ostream& operator << ( ostream& s, const bitGene & _inGen ) {
// --	to print a rawGene to stdout
// -----------------------------------------------------------------------

    for ( genLength_t i = 0; i < _inGen.getLength(); i ++ ) 
	s << _inGen[i] << " ";
	
    return s;
}

