// -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; c++-electric-colon: 1;  comment-column: 35 -*-


//			GENSGAV.HPP
// DESCRIPTION: same as gensga.hpp, but with variable range for each locus
//		in the cromosome.


#ifndef __GENSGA_HPP
#define __GENSGA_HPP

#ifndef __BCPLUSPLUS__
#ident "@(#) gensga.hpp -- by J. J. Merelo"
#endif

#include "genvar.hpp"
#include "weight.hpp"


template <class returnType>
class genSGA: public genVar {

  public:


    // constructors and destructors.
    virtual ~genSGA () {}
    
    genSGA() {};

    genSGA(genLength_t _numAlleles, 
	   unsigned _sizeAllele, 
	   returnType _rgStart, 
	   returnType _rgEnd)
      :genVar( _numAlleles,  _sizeAllele), 
    rangeStart( _rgStart), 
    range( _rgEnd - _rgStart) {
	if ( _sizeAllele >= sizeof( returnType ) ) 
	    cerr << "More bytes in gen than in the type it returns\n";
    };
				   // Main constructor
 
    genSGA(genLength_t _numAlleles, 
	   unsigned _sizeAllele, 
	   returnType *_rgStart, 
	   returnType *_rgEnd) {}; // Dummy
	    
	    
    // Unary constructors, from only 1 parent
    genSGA(const genSGA &_genSGA, // creates a new genome using one
	   genOpMode _modeOp,  // of 3 modes; using _geneIdx if
	   genLength_t _geneIdx = 0, // needed and _mutationRate
	   mutRate_t _mutationRate = 0,
	   genLength_t _geneIdx2 = 0)
      :genVar( _genSGA, _modeOp, _geneIdx, _mutationRate, _geneIdx2 ), 
    rangeStart( _genSGA.rangeStart ), 
    range( _genSGA.range ) {};	

    genSGA( const genSGA &_genSGA,
	   mutRate_t _mutationRate )
      :genVar( _genSGA, _mutationRate ),
    rangeStart( _genSGA.rangeStart ), 
    range( _genSGA.range ) {};


//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//               Binary Ctors
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

    genSGA(const genSGA &genSGA1, 
	   const genSGA &genSGA2, 
	   genLength_t _startPoint, genLength_t _endPoint, 
	   mutRate_t _mutationRate)
      :genVar( genSGA1, genSGA2, _startPoint, _endPoint, _mutationRate ),
    rangeStart( genSGA1.rangeStart ), range( genSGA1.range ) {}; 
				   // 2-pt xOver + mutation

    genSGA(const genSGA &_genSGA1, 
	    const genSGA &_genSGA2, 
	    xOverRate_t _xOverRate, 
	    mutRate_t _mutationRate = 0) 
      :genVar(_genSGA1, _genSGA2,_xOverRate, _mutationRate ),
    rangeStart( _genSGA1.rangeStart ), range( _genSGA1.range ) {}; 
				   // Uniform xOver + opt. mutation


    genSGA(const genSGA& _inGen )
      :genVar( _inGen ),  
    rangeStart( _inGen.rangeStart ), range( _inGen.range ){};	     
				   // copy constructor
  

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- 
//   Common functions to all the "gene" hierarchy
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

    // "info" functions, and overloaded operators
    returnType	operator [] (genLength_t _idx) const;

    // getLength is directly inherited from genVar

    // conversion operators
    operator returnType * ( void ) const;
    operator Weight<returnType> (void) const;
    Weight<returnType> extractVect(genLength_t _idx1, 
				   genLength_t _idx2) const;

    // Particular info function
    // now functions for fancy Gray decoding
    returnType	deGray	    ( genLength_t _idx ) const;
   
    // Friend declaration
friend ostream& operator << ( ostream& s, const genVar & _inGen );
    

private:
    returnType rangeStart, range; // beginning of range and range	

};

#include "gensga.icc"		   // templates and inlines, hidden
				   // from prying eyes

#endif				   // __GENSGA_HPP
