// -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; c++-electric-colon: t; comment-column 35 -*-

// ----------------------------- POPS.HPP -----------------------------------
// DESCRIPTION : class to handle a population of fixed-length genomes 
//               using a SGA
//		 Several classes inherit from it, depending on the type 
//               of genome (or at least they should
//--------------------------------------------------------------------------

// REFERENCES	: Goldberg's book, JJ's thesis, and many other things.


#ifndef __POPS_HPP
#define __POPS_HPP

#ifndef __BCPLUSPLUS__
#ident "@(#) pops.hpp -- By JJ Merelo "
#endif

#include "general.hpp"				  // for swap

template <class returnType>
class popS{

public:

    // constructors and destructors
    popS( unsigned _popSize,  mutRate_t _mutRate );
	
    virtual ~popS();    
	    
    // "peeping Tom" functions
    unsigned	getPopSize	( void) const { return popSize; };
    returnType	getFitness	( unsigned _idx ) const { 
      return fitness[_idx];
    };
        
    // assignation functions
    void	setFitness(unsigned _idx, 
			   returnType _value ) { 
      (_idx < popSize)?(fitness[_idx] = _value):0;
//      cerr << "Fitness " << _idx << " =  " << fitness[_idx] << endl;
    }
       
       
    // "best guy" functions
    unsigned	index2rank( unsigned _idx ) const { // return the genome whose 
	return rank[_idx];			    // fitness is in _idx_ position
    }
    unsigned	index2best( void) const { return rank[0];}
				                  // same, with the best
    returnType	bestFitness( void ) const { return fitness[rank[0]]; };


    // unary operator functions
    void        apply1transpose( float _trFreq ) { transposeFreq = _trFreq; };
    void        apply1clone( float _cloneFreq ) { cloneFreq = _cloneFreq; };
    void        apply1increment( incRate_t _incFreq ) { incFreq = _incFreq; };
    void        apply1decrement( killRate_t _killFreq) {killFreq = _killFreq;};
    void        apply1randinc( incRate_t _rIncFreq ) { rIncFreq = _rIncFreq;};

protected:
    returnType*		fitness; // array of fitness for each individual
    unsigned *		rank;	 // rank according to fitness
    unsigned		popSize; // population size
    mutRate_t		mutationRate; // rate of mutation
    incRate_t           incFreq,
                        rIncFreq;      // variable-length op rate
    killRate_t          killFreq;      // Decrement-op rate
    double              transposeFreq; // frequency of transposition operator
    double              cloneFreq;     // frequency for cloning operator

    // selection and other population functions
    void	fastRank(); // orders according to fitness
    void        Rank();	    // the same, but slow

					    
    virtual void    createRWPool( void) = 0; // creates roulette wheel pool,
					     // according to fitness
    virtual void    reproducePool( void) = 0;	  // reproduces from genePool
    virtual void    eliteRep( float _elitePerc ) = 0;   
						  // steady state reproduction
    virtual void    createTourPool( unsigned _tournSize ) = 0;
                                                  // tournament selection
    
    // genetic operators
    virtual void    Clone( unsigned _winner,  unsigned _newPos ) = 0;	    
				// clones with mutation
    virtual void Mate(unsigned _par1, // mates them with given mutationRate
		      unsigned _par2, // places result in the population
		      unsigned _sib ) = 0;  
		
    virtual void eliteMate(unsigned _par1, // mates them with given mutationRate
			   unsigned _par2, // places result in the population
			   unsigned _sib ) = 0;	// does not use genePool

private:
    void       quickSort(int _idx0, 
			 int _idxN ); // orders using quickSort 
	   
};

#include "pops.icc"

#endif				// __POPS_HPP
