// -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; c++-electric-colon: t; comment-column: 35 -*-


// --------------------------- POPSGAR.HPP -----------------------------------
// DESCRIPTION : class to handle a population of fixed-length genomes using a
//               SGA
//		 This version uses 2 templates, since I learned how 
//               to handle them



#ifndef __POPSGAR_HPP
#define __POPSGAR_HPP

#ifndef __BCPLUSPLUS__
#ident "@(#) popsgar.hpp -- By JJ Merelo "
#endif


#include "bitgene.hpp"
#include "gensgav.hpp"
#include "gensga.hpp"
#include "pops.hpp"			     // base class

enum reprodMode { ELITE, TOURNAMENT, ROULWHEEL }; // Major reproduction modes
	

template <class returnType, class genType>
class popSGAr: public popS<returnType>{
        
public:
    
    
    // constructors and destructors
    popSGAr(unsigned _popSize,  
	    mutRate_t _mutationRate,
	    genLength_t _numParams, 
	    unsigned _sizeParams, 
	    returnType* _rgStart = 0,
	    returnType* _rgEnd = 0); // interpreted in different
				     // ways by different constructors
    popSGAr(unsigned _popSize,  
	    mutRate_t _mutationRate,
	    genLength_t _numParamsMin,
	    genLength_t _numParamsMax,
	    unsigned _sizeParams, 
	    returnType* _rgStart = 0,
	    returnType* _rgEnd = 0); // Same as before, with var-length
    
    popSGAr(unsigned _popSize,  
	    mutRate_t _mutationRate,
	    genLength_t _numParams, 
	    unsigned _sizeParams, 
	    returnType _rgStart,
	    returnType _rgEnd);	   // To use with genSGA

    popSGAr(unsigned _popSize,  
	    mutRate_t _mutationRate,
	    genLength_t _numParamsMin, 
	    genLength_t _numParamsMax,
	    unsigned _sizeParams, 
	    returnType _rgStart,
	    returnType _rgEnd);	   // To use with genSGA
    
	    
    virtual ~popSGAr();    
    
    //________________________________________________________________
    
    // look inside functions
    const genType * operator [] ( unsigned _genomeIdx ) const {
	return pop[_genomeIdx];
    };
    
    const genType& bestIndiv( void ) const { return (*pop[ rank[0]]); };
    
    // reproduction overloaded functions
    void            newGeneration(reprodMode _mode,
				  unsigned _operand); 
				   // reproduction tournament or RW
    void            newGeneration(reprodMode _mode,
				  float _operand ); // elite reproduction

    // friend declarations
friend ostream& operator << (ostream& s, 
			     const popSGAr<returnType, genType> & _inPop );

  protected:
				    
    // Roulette wheel and tournament selection functions		    
    virtual void    createRWPool( void); // creates roulette wheel pool, 
					 // according to fitness
    virtual void    createTourPool( unsigned _tournSize );
				   // tournament selection
    virtual void    reproducePool( void); // reproduces from genePool
    
    // And steady-state functions
    virtual void    eliteRep( float _elitePerc );   
				   // steady state reproduction
    
    
    // genetic operators
    virtual void Clone( unsigned _winner,  unsigned _newPos );	    
				   // clones with mutation
    virtual void Mate(unsigned _par1, // mates them with given mutationRate
		      unsigned _par2, // places result in the population
		      unsigned _sib );  
		
    virtual void eliteMate(
		unsigned _par1,	   // mates them with given mutationRate
		unsigned _par2,	   // places result in the population
		unsigned _sib );   // does not use genePool

//___________________________________________________________________

  private:

    genType **pop, **genePool;	   // current population and genePool
    

    // private funcs
    void allocMem( unsigned _popSize);
	   
};


#include "popsgar.icc"

#endif
