// -*- mode: c++; c-indent-level: 4; c++-member-init-indent: 8; c++-electric-colon: t; comment-column: 35;  -*-

#ifndef __SCANFILE_HPP
#define __SCANFILE_HPP

#include "paramarr.hpp"		// includes general & popsgar

#include <ctype.h>		// for touper()

// main access functions

// 3- without ranges, for bitgene.
void scanFile3(char* _fileN,		
	       char* _inputFile, 
	       char* _outputFile,
	       unsigned& _popSize,
	       mutRate_t& _mutRate,
	       unsigned& _gens,
	       enum reprodMode & _repType,  
	       unsigned& _uRepArg,
	       float&    _fRepArg,
	       mybool& _gnuplot,
	       unsigned& _outputInterv,
	       mybool& _outFitness,
	       mybool& _outBest );

void scanBest( const char* _inStr, mybool & _fit, mybool & _best );

void scanRep( const char* _inStr, 
	     enum reprodMode& _rep,
	     float& _fArg,
	     unsigned& _uArg );

//                      TEMPLATES
//--------------------------------------------------------------------------
template <class T>
void scanFile(  T* _rgStart, T* _rgEnd,
	      char* _fileN,		
	      char* _inputFile, 
	      char* _outputFile,
	      unsigned& _popSize,
	      mutRate_t& _mutRate,
	      unsigned& _gens,
	      unsigned& _locusSize,
	      enum reprodMode & _repType,  
	      unsigned& _uRepArg,
	      float&    _fRepArg,
	      const genLength_t _genSize,
	      mybool& _gnuplot,
	      unsigned& _outputInterv,
	      mybool& _outFitness,
	      mybool& _outBest )  

// -- scan files for commands of the following form
//    description parameter
//    description is a symbolic name; while parameter is a string or numeric
//    value. All values are returned in vars passed by reference
//    It's quite ad hoc, but it works.
//    Actually, it looks only for the first letter. So, no more than 24
//    parameters.
// -------------------------------------------------------------------------
{
    paramArr params( _fileN );

    // output options
    char auxfv[MAXPARAMSIZE];
    params.getStr( 'B', auxfv, MAXPARAMSIZE );
    scanBest( auxfv, _outFitness, _outBest );

    // End range
    params.getStr( 'E', auxfv, MAXPARAMSIZE );
    istrstream istr( auxfv, MAXPARAMSIZE );
    for ( unsigned i = 0; i < _genSize; i ++ ) 
      istr >> _rgEnd[i];

    params.getStr( 'F', _outputFile, MAXFILENAME );
    _gens = params.getInt( 'G' );
    params.getStr('I', _inputFile, MAXFILENAME );
    _locusSize = params.getInt( 'L' );
    _mutRate = params.getFloat( 'M' );
    _outputInterv = params.getInt( 'O' );
    _popSize    = params.getInt( 'P' );
    
    // reproduction type
    char tmpRepStr[MAXPARAMSIZE];
    params.getStr( 'R', tmpRepStr, MAXPARAMSIZE );
    scanRep( tmpRepStr, _repType, _fRepArg, _uRepArg );

    params.getStr( 'S', auxfv, MAXPARAMSIZE );
    istrstream istr2( auxfv, MAXPARAMSIZE );
    for ( i = 0; i < _genSize; i ++ ) 
      istr2 >> _rgStart[i];

    // use gnuplot or not
    char tmpWinGnu[MAXPARAMSIZE];
    params.getStr( 'W', tmpWinGnu, MAXPARAMSIZE );
    _gnuplot = (toupper(tmpWinGnu[0]) == 'Y')?TRUE:FALSE;

    // some error checking
    for (  i = 0; i < _genSize; i ++ )
      if ( _rgStart[i] == _rgEnd[i] ) {
	  cerr << "Parameter # " << i << "has range 0\n";
	  exit(1);
      }
};


//--------------------------------------------------------------------------
template <class T>
void scanFile2(  T& _rgStart, T& _rgEnd,
	      char* _fileN,		
	      char* _inputFile, 
	      char* _outputFile,
	      unsigned& _popSize,
	      mutRate_t& _mutRate,
	      unsigned& _gens,
	      unsigned& _locusSize,
	      enum reprodMode & _repType,  
	      unsigned& _uRepArg,
	      float&    _fRepArg,
	      mybool& _gnuplot,
	      unsigned& _outputInterv,
	      mybool& _outFitness,
	      mybool& _outBest ) { 

// -- scan files for commands of the following form
//    description parameter
//    description is a symbolic name; while parameter is a string or numeric
//    value. All values are returned in vars passed by reference
//    It's quite ad hoc, but it works.
//    Actually, it looks only for the first letter. So, no more than 24
//    parameters.
// -------------------------------------------------------------------------


    paramArr params( _fileN );

    // output options
    char auxfv[MAXPARAMSIZE];
    params.getStr( 'B', auxfv, MAXPARAMSIZE );
    scanBest( auxfv, _outFitness, _outBest );

    params.getStr( 'E', auxfv, MAXPARAMSIZE );
    istrstream istr( auxfv, MAXPARAMSIZE );
    istr >> _rgEnd;		// all this to account for different types

    params.getStr( 'F', _outputFile, MAXFILENAME );
    _gens = params.getInt( 'G' );
    params.getStr('I', _inputFile, MAXFILENAME );
    _locusSize = params.getInt( 'L' );
    _mutRate = params.getFloat( 'M' );
    _outputInterv = params.getInt( 'O' );
    _popSize    = params.getInt( 'P' );
    
    // reproduction type
    char tmpRepStr[MAXPARAMSIZE];
    params.getStr( 'R', tmpRepStr, MAXPARAMSIZE );
    scanRep( tmpRepStr, _repType, _fRepArg, _uRepArg );

    // end range
    params.getStr( 'S', auxfv, MAXPARAMSIZE );
    istrstream istr2( auxfv, MAXPARAMSIZE );
    istr2 >> _rgStart;		// all this to account for different types

    // use gnuplot or not
    char tmpWinGnu[MAXPARAMSIZE];
    params.getStr( 'W', tmpWinGnu, MAXPARAMSIZE );
    _gnuplot = (toupper(tmpWinGnu[0]) == 'Y')?TRUE:FALSE;

    // some error checking
    if ( _rgStart == _rgEnd ) {
	cerr << "Chromosome has range 0 in scanfile\n";
	exit(1);
    }
};


#endif				// __SCANFILE_HPP
